( function( $ ) {

    let $document = $(document);

    let form_id;
    let container;
    let credChanged=0;

    if (typeof elementor !== 'undefined'){
        elementor.hooks.addAction( 'panel/open_editor/widget/form', ( panel, model, view )=>{
            container = view.getContainer();
            form_id = view.model.id;
        });
    }

    $document.on('click', '.elementor_ap_google_sheet_credentials_validate button', function (e) {
        e.preventDefault();
        const $button = $(this);
        let client_id = $('.elementor_ap_google_sheet_client_id input').val();
        let client_secret = $('.elementor_ap_google_sheet_client_secret input').val();
        let redirect_uri =  $('.ap-google-sheet-redirect-uri').text();
        let scopes = 'https://www.googleapis.com/auth/spreadsheets';
        let grant_type = 'authorization_code';
        let prompt = 'consent'
        let url = 'https://accounts.google.com/o/oauth2/v2/auth?client_id='+client_id+'&redirect_uri='+redirect_uri+'&scope='+scopes+'&access_type=offline&include_granted_scopes=true&state=state_parameter_passthrough_value&response_type=code&prompt='+prompt;

        loading($button);
        oauthSignIn(url).then( function (code) {
            if(code){
                $.ajax({
                    type: 'POST',
                    url: "https://accounts.google.com/o/oauth2/token",
                    contentType: 'application/x-www-form-urlencoded; charset=utf-8',
                    dataType: 'json',
                    data: {
                        client_id: client_id,
                        client_secret: client_secret,
                        code: code,
                        redirect_uri: redirect_uri,
                        grant_type: grant_type,
                    },
                    success: function (data) {
                        if(data.refresh_token){
                            if( typeof container !== 'undefined' ){
                                $e.run( 'document/elements/settings', {
                                    container: container,
                                    settings: {
                                        [ 'ap_google_sheet_access_token' ]: data.access_token,
                                        [ 'ap_google_sheet_refresh_token' ]: data.refresh_token,
                                        [ 'ap_google_sheet_expires_in' ]: Math.round(new Date().getTime() / 1000) + data.expires_in
                                    }
                                } );
                            }
                            else{
                                $('#elementor_ap_google_sheet_access_token').val(data.access_token);
                                $('#elementor_ap_google_sheet_refresh_token').val(data.refresh_token);
                                $('#elementor_ap_google_sheet_expires_in').val(Math.round(new Date().getTime() / 1000) + data.expires_in );
                            }
                            success($button);
                            credChanged = 1;
                        }
                    }
                });
            }
            else{
                error($button);
            }
        });

    });

    $document.on('change', '[data-setting="ap_google_sheet_url"]', function (e){

        $e.run( 'document/elements/settings', {
            container: container,
            settings: {
                [ 'ap_google_sheet_id' ]: new RegExp("/spreadsheets/d/([a-zA-Z0-9-_]+)").exec($('[data-setting="ap_google_sheet_url"]').val())[1]
            }
        } );
    });

    const loading = function ($button){
        $button.removeClass('success error').addClass('loading');
        $('i', $button).removeClass('fa-check fa-close').addClass('fa-spin');
    }

    const success = function ($button){
        $button.removeClass('loading success').addClass('success');
        $('i', $button).removeClass('fa-refresh fa-spin fa-close').addClass('fa-check').css('color','green');
    }

    const error = function ($button){
        $button.removeClass('loading success').addClass('error');
        $('i', $button).removeClass('fa-refresh fa-spin fa-check').addClass('fa-close').css('color','red');
    }

    const oauthSignIn = function(url) {
        let dfd = $.Deferred();
        const h = 600;
        const w = 450;
        const y = window.top.outerHeight / 2 + window.top.screenY - ( h / 2);
        const x = window.top.outerWidth / 2 + window.top.screenX - ( w / 2);
        let win = window.open(url, "_blank", `width=${w}, height=${h}, top=${y}, left=${x}`);

        let timer = window.setInterval(function() {
            try {
                let currentUrl = win.document.URL;
                if (currentUrl.indexOf('code') !== -1) {
                    window.clearInterval(timer);
                    win.close();
                    let code = getTokenValue(currentUrl, 'code')
                    dfd.resolve(code);
                }
                else if (win.document.URL.indexOf('error') !== -1) {
                    window.clearInterval(timer);
                    win.close();
                    dfd.resolve();
                }
            } catch(e) {
            }
            if(win.closed){
                window.clearInterval(timer);
                dfd.resolve();
            }
        },100);
        return dfd;
    }

    const getTokenValue = function(url, name) {
        // let dfd = $.Deferred();
        name = name.replace(/[[]/,"\[").replace(/[]]/,"\]");
        let regexS = "[\?&]"+name+"=([^&#]*)";
        let regex = new RegExp( regexS );
        let results = regex.exec( url );
        return decodeURIComponent(results[1]);
    };

    window.onbeforeunload = function confirmExit() {
        if (credChanged === 1) {
            return "New information not saved. Do you wish to leave the page?";
        }
    }

    $document.on('click', '#elementor-panel-saver-button-publish-label', function() {
        credChanged = 0;
    });

    $("input[name='submit']").on('click', function() {
        credChanged = 0;
    });


} )( jQuery );
