"use strict";

( function( $ ) {

    let $document = $(document);

    let container, post_id, form_id, cache = {}, credChanged = 0, customApiData = '';

    if (typeof elementor !== 'undefined'){
        elementor.hooks.addAction( 'panel/open_editor/widget/form', ( panel, model, view )=>{
            container = view.getContainer();
            form_id = view.model.id;
            post_id = elementorFrontend.config.post.id;
        });
    }

    $document.on('click', '.elementor_ap_zoho_crm_credentials_validate button', function (e) {
        e.preventDefault();

        let $button = $(this),
            client_id = $('.elementor_ap_zoho_crm_client_id input').val(),
            redirect_uri = $('.ap_zoho_crm_redirect_uri').text(),
            authUri = 'https://accounts.zoho.com/oauth/v2/auth?' +'access_type=offline' + '&response_type=code' + '&prompt=' + 'consent' + '&scope=' + 'ZohoCRM.settings.modules.READ,ZohoCRM.settings.fields.read,ZohoCRM.modules.CREATE' + '&client_id=' + client_id + '&redirect_uri=' + redirect_uri;

        loading($button);

        oauthSignIn(authUri)
            .then( function( responseUri ){
                let source = $('[data-setting="ap_zoho_crm_credentials_source"]').val() || 'default';
                let action = 'ap_zoho_crm_credentials_validate';
                let client_secret = $('.elementor_ap_zoho_crm_client_secret input').val();
                let code = extractParameterValue( responseUri, 'code');
                let accounts_server = extractParameterValue( responseUri, 'accounts-server');
                return credentialsValidate( action, source, client_id, client_secret, code, accounts_server );
            }, function (){
                error($button);
            })
            .then( function ( response ){
                try{
                    if(response.data === 'success'){
                        success($button);
                        credChanged = 1;
                    }
                    else if( response.data.access_token !== ''){
                        success($button);

                        $e.run( 'document/elements/settings', {
                            container: container,
                            settings: {
                                [ 'ap_zoho_crm_access_token' ]: response.data.access_token,
                                [ 'ap_zoho_crm_refresh_token' ]: response.data.refresh_token,
                                [ 'ap_zoho_crm_expires_in' ]: Math.round(new Date().getTime() / 1000) + response.data.expires_in,
                                [ 'ap_zoho_crm_api_domain' ] : response.data.api_domain,
                                [ 'ap_zoho_crm_accounts_server' ]: response.data.accounts_server,
                            }
                        });

                        getModuleNames($('[data-setting="ap_zoho_crm_module_field"]'), 'custom' ).then(function( options ){
                            populateSingleField('ap_zoho_crm_module', options);
                        });
                    }
                    else{
                        error($button)
                    }
                }catch (e){
                    error($button);
                }
            });
    });

    $document.on('change', '[data-setting="ap_zoho_crm_credentials_source"]', function (){
        cache.modules = null;
        cache.fields = null;
        $('.elementor-repeater-row-controls').removeClass('editable');
        let $select = $('[data-setting="ap_zoho_crm_module"]').html('');
        let apiSource = $(this).val();

        getModuleNames($select, apiSource).then(function( options ){
            populateSingleField('ap_zoho_crm_module', options);
        });
    });

    $document.on('change', '[data-setting="ap_zoho_crm_module"]', function (){
        $('.elementor-repeater-row-controls').removeClass('editable');
        cache.fields = null;
        $('[data-setting="ap_zoho_crm_module_field"]').html('');
    });

    $document.on('mouseup', '.elementor-control-ap_section_zoho_crm', function(e){
        // If section opened
        if ( $(this).hasClass('elementor-open') === false){
            cache.formFields = null;

            // If our target Element is ready
            elementReady('[data-setting="ap_zoho_crm_module"]').then((element)=>{
                let $select = $(element);
                let apiSource = container.settings.attributes['ap_zoho_crm_credentials_source'];
                getModuleNames($select, apiSource).then(function( options ){
                    populateSingleField('ap_zoho_crm_module', options);
                });
            });
        }
    });

    $document.on('mouseup', '.elementor-control-ap_zoho_crm_fields_mapping .elementor-repeater-row-item-title', function(e) {
        let $select = $(this).parent().next().find('[data-setting="ap_zoho_crm_module_field"]');
        let apiSource = container.settings.attributes['ap_zoho_crm_credentials_source'];
        let moduleName = container.settings.attributes['ap_zoho_crm_module'];
        getModuleFields($select,apiSource, moduleName).then(function( options ){
            populateRepeaterField($select, 'ap_zoho_crm_fields_mapping', 'ap_zoho_crm_module_field', options);
        });

        let $formFieldSelect = $(this).parent().next().find('[data-setting="ap_zoho_crm_form_field"]');
        getFormFields( $formFieldSelect ).then(function (options){
            populateRepeaterField($formFieldSelect, 'ap_zoho_crm_fields_mapping', 'ap_zoho_crm_form_field', options);
        });
    });

    $document.on('click', '.elementor-control-ap_zoho_crm_fields_mapping .elementor-repeater-add', function(e) {
        let $select = $(this).parent().prev().find('[data-setting="ap_zoho_crm_module_field"]').last();
        let apiSource = container.settings.attributes['ap_zoho_crm_credentials_source'];
        let moduleName = container.settings.attributes['ap_zoho_crm_module'];
        getModuleFields($select,apiSource, moduleName).then(function( options ){
            populateRepeaterField($select, 'ap_zoho_crm_fields_mapping', 'ap_zoho_crm_module_field', options);
        });

        let $formFieldSelect = $(this).parent().prev().find('[data-setting="ap_zoho_crm_form_field"]').last();
        getFormFields( $formFieldSelect ).then(function (options){
            populateRepeaterField($formFieldSelect, 'ap_zoho_crm_fields_mapping', 'ap_zoho_crm_form_field', options);
        });
    });

    $document.on('click', '#elementor-settings-form  input[name="submit"]', function() {
        credChanged = 0;
    });

    function oauthSignIn( uri ) {
        let dfd = $.Deferred();
        const h = 600;
        const w = 450;
        const y = window.top.outerHeight / 2 + window.top.screenY - ( h / 2);
        const x = window.top.outerWidth / 2 + window.top.screenX - ( w / 2);
        let popup = window.open(uri, "_blank", `width=${w}, height=${h}, top=${y}, left=${x}`);

        let timer = window.setInterval(function() {
            try {
                let responseUri = popup.document.URL;
                if (responseUri.indexOf('code') !== -1) {
                    window.clearInterval(timer);
                    popup.close();
                    dfd.resolve(responseUri);
                    return;
                }
                else if (responseUri.indexOf('error') !== -1) {
                    window.clearInterval(timer);
                    popup.close();
                    dfd.reject();
                    return;
                }
            }
            catch(e) {
            }

            if(popup.closed){
                window.clearInterval(timer);
                dfd.reject();
            }
        },100);

        return dfd;
    }

    function extractParameterValue(url, parameter) {
        parameter = parameter.replace(/[[]/,"\[").replace(/[]]/,"\]");
        let regexS = "[\?&]"+parameter+"=([^&#]*)";
        let regex = new RegExp( regexS );
        let results = regex.exec( url );
        return decodeURIComponent(results[1]);
    }

    function credentialsValidate(action, source, client_id, client_secret, code, accounts_server){
        return $.ajax({
            url: ajaxurl,
            type: 'post',
            dataType: 'json',
            data:{
                action: action,
                source: source,
                client_id: client_id,
                client_secret: client_secret,
                code: code,
                accounts_server: accounts_server
            }
        });
    }

    function loading($button){
        $button.removeClass('success error').addClass('loading');
        $('i', $button).removeClass('fa-check fa-close').addClass('fa-spin');
    }

    function success($button){
        $button.removeClass('loading success').addClass('success');
        $('i', $button).removeClass('fa-refresh fa-spin fa-close').addClass('fa-check').css('color','green');
    }

    function error($button){
        $button.removeClass('loading success').addClass('error');
        $('i', $button).removeClass('fa-refresh fa-spin fa-check').addClass('fa-close').css('color','red');
    }

    function elementReady(selector) {
        return new Promise((resolve, reject) => {
            let el = document.querySelector(selector);
            if (el) {resolve(el);}
            new MutationObserver((mutationRecords, observer) => {
                // Query for elements matching the specified selector
                Array.from(document.querySelectorAll(selector)).forEach((element) => {
                    resolve(element);
                    //Once we have resolved we don't need the observer anymore.
                    observer.disconnect();
                });
            })
                .observe(document.documentElement, {
                    childList: true,
                    subtree: true
                });
        });
    }

    function getModuleNames( $select, apiSource){
        let def = $.Deferred();

        if($select.has('option').length){
            return def.reject();
        }

        if ( cache.modules ){
            return def.resolve(cache.modules);
        }

        if(apiSource === 'custom' && customApiData === ''){
            let settings = container.settings.attributes;

            // User is trying set custom API for first time
            if( settings['ap_zoho_crm_client_id'] === '' && settings['ap_zoho_crm_client_secret'] === ''){
                return def.reject();
            }

            customApiData = JSON.stringify({
                form_id : form_id,
                post_id : post_id,
                expires_in : settings['ap_zoho_crm_expires_in'],
                client_id : settings['ap_zoho_crm_client_id'],
                client_secret : settings['ap_zoho_crm_client_secret'],
                access_token : settings['ap_zoho_crm_access_token'],
                refresh_token : settings['ap_zoho_crm_refresh_token'],
                accounts_server : settings['ap_zoho_crm_accounts_server'],
                api_domain : settings['ap_zoho_crm_api_domain']
            });
        }

        $.ajax({
            url: ajaxurl,
            type: 'POST',
            dataType: 'json',
            data:{
                action: 'ap_zoho_crm_get_module_names',
                source: apiSource,
                customApiData: customApiData
            },
            beforeSend: function(){
                $select.parent().before('<i class="eicon-spinner eicon-animation-spin"></i>');
            },
            success: function (response){
                cache.modules = response.data;
                $select.parent().prev().remove();
                return def.resolve(response.data);
            }
        })

        return def.promise();
    }

    function getModuleFields($select, apiSource, moduleName){
        let def = $.Deferred();

        if($select.has('option').length){
            return def.reject();
        }

        if ( cache.fields ){
            return def.resolve(cache.fields);
        }

        if(apiSource === 'custom' && customApiData === ''){
            let settings = container.settings.attributes;
            customApiData = JSON.stringify({
                form_id : form_id,
                post_id : post_id,
                expires_in : settings['ap_zoho_crm_expires_in'],
                client_id : settings['ap_zoho_crm_client_id'],
                client_secret : settings['ap_zoho_crm_client_secret'],
                access_token : settings['ap_zoho_crm_access_token'],
                refresh_token : settings['ap_zoho_crm_refresh_token'],
                accounts_server : settings['ap_zoho_crm_accounts_server'],
                api_domain : settings['ap_zoho_crm_api_domain']
            });
        }

        $.ajax({
            url: ajaxurl,
            type: 'POST',
            dataType: 'json',
            data:{
                action: 'ap_zoho_crm_get_module_fields',
                source: apiSource,
                module: moduleName,
                customApiData: customApiData
            },
            beforeSend: function(){
                $select.parent().before('<i class="eicon-spinner eicon-animation-spin"></i>');
            },
            success: function (response){
                cache.fields = response.data;
                $select.parent().prev().remove();
                return def.resolve(response.data);
            }
        })

        return def.promise();
    }

    function getFormFields( $select ){
        let def = $.Deferred();

        // Useful for stopping further execution when a field is closed on click
        if($select.has('option').length){
            return def.reject();
        }

        if ( cache.formFields ){
            return def.resolve(cache.formFields);
        }

        let fields = container.settings.attributes['form_fields'].models;
        let item=1, options = '<option value="meta">Meta Data</option>';

        $.each(fields, function( index ){
            options += '<option value='+fields[index].attributes.custom_id+'>'+ (fields[index].attributes.field_label || fields[index].attributes.placeholder || 'Item #'+item) +'</option>';
            item++;
        });

        cache.formFields = options;
        return def.resolve(options);
    }

    function populateRepeaterField( $select, repeaterName, fieldName, options){
        let _id = $select.closest('.elementor-repeater-fields').find('[data-setting="_id"]').val();
        let selected = container.settings.attributes[repeaterName].models.filter(model=>model.attributes._id === _id)[0].attributes[fieldName];
        $select.html(options).val(selected);
    }

    // TODO: replace populateRepeaterField() with below
    //  https://github.com/elementor/elementor/issues/12666
    // function updateOptionsInRepeater( repeaterName, innerControlName, options ) {
    //     // get current panel view
    //     const editor = elementor.getPanelView().getCurrentPageView();
    //     // get repeater cid
    //     let cid = editor.collection.findWhere({name: repeaterName} ).cid;
    //     // get repeater view
    //     const repeaterView = editor.children.findByModelCid( cid );
    //     // set the options of an inner control
    //     if ( repeaterView && repeaterView.model.attributes.fields[ innerControlName ] ) {
    //         repeaterView.model.attributes.fields[ innerControlName ].options = options;
    //         repeaterView.render();
    //     }
    // }

    function populateSingleField( fieldName, options){
        let editor = elementor.getPanelView().getCurrentPageView();
        let model =  editor.collection.findWhere({name: fieldName} );
        model.set('options', options);
        let cid = model.cid;
        let view = editor.children.findByModelCid(cid);
        view.render();
    }

    window.onbeforeunload = function confirmExit() {
        if (credChanged === 1) {
            return "New information not saved. Do you wish to leave the page?";
        }
    }

    // {{{apZohoCrmFieldsMappingTitle()}}}
    window.apZohoCrmFieldsMappingTitle = function( moduleFieldValue ) {
        return moduleFieldValue.replace(/_/g, ' ');
    }

} )( jQuery );
